@@ -1,3 +1,5 @@
+require 'timeout'
+
 require 'active_ldap/schema'
 require 'active_ldap/entry_attribute'
 require 'active_ldap/ldap_error'
@@ -12,6 +14,7 @@ module ActiveLdap
         :host,
         :port,
         :method,
+        :tls_options,
         :timeout,
         :retry_on_timeout,
         :retry_limit,
@@ -328,7 +331,7 @@ module ActiveLdap
         n_retries = 0
         retry_limit = options[:retry_limit] || @retry_limit
         begin
-          do_in_timeout(@timeout, &block)
+          Timeout.timeout(@timeout, &block)
         rescue Timeout::Error => e
           @logger.error {_('Requested action timed out.')}
           if @retry_on_timeout and (retry_limit < 0 or n_retries <= retry_limit)
@@ -344,10 +347,6 @@ module ActiveLdap
         end
       end
 
-      def do_in_timeout(timeout, &block)
-        Timeout.alarm(timeout, &block)
-      end
-
       def sasl_bind(bind_dn, options={})
         # Get all SASL mechanisms
         mechanisms = operation(options) do
@@ -601,9 +600,13 @@ module ActiveLdap
             raise
           rescue => detail
             @logger.error do
-              _("Reconnect to server failed: %s\n" \
+              _("Reconnect to server failed: %s: %s\n" \
                 "Reconnect to server failed backtrace:\n" \
-                "%s") % [detail.message, detail.backtrace.join("\n")]
+                "%s") % [
+                detail.class,
+                detail.message,
+                detail.backtrace.join("\n"),
+              ]
             end
             # Do not loop if forced
             raise ConnectionError, detail.message if force
